//@author: ft
//@description: Draws particlesGPU getting info from data texture
//@tags: 3d 2d particles
//@credits: dottore

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD ;        //the models world matrix
float4x4 tWi: WORLDINVERSE ;
float4x4 tWVPi: WORLDVIEWPROJECTIONINVERSE ;
float4x4 tWVi: WORLDVIEWINVERSE ;
float4x4 tVi: VIEWINVERSE ;
float4x4 tPi: PROJECTIONINVERSE ;
float4x4 tV: VIEW ;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION ;   //projection matrix as set via Renderer (EX9)
float4x4 tVP: VIEWPROJECTION ;


//texture Color
texture TexCol <string uiname="Particle Texture";>;
sampler SampCol = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (TexCol);          //apply a texture to the sampler
    MipFilter = Linear;         //sampler states
    MinFilter = Linear;
    MagFilter = Linear;
};
float4x4 tTexCol: TEXTUREMATRIX <string uiname="Texture Transform";>;

float Size <string uiname="Particle Size";> = 1 ;

int ViewportHeight <string uiname="Renderer Backbuffer Height";> = 480;

static const float PI = 3.14159265f;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2psPointSprite
{
    float4 Pos : POSITION ;
    float2 TexCdCol : TEXCOORD0 ;
	float Size : PSIZE0;  //////////////////  this allows usage of pointsize
};

struct vs2psQuad
{
    float4 Pos : POSITION ;
    float2 TexCdCol : TEXCOORD0 ;
};


// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2psPointSprite VSPointSprite(
		float4 Pos : POSITION,
		float4 TexCd : TEXCOORD0
	) 
{
    //inititalize all fields of output struct with 0
    vs2psPointSprite Out = (vs2psPointSprite)0;
	
    //apply the tW (points at the mesh position)
    Pos.xyz = mul(Pos, tW);
    //then apply the tVP
    Out.Pos = mul(Pos, tVP);

	//doesn't seem to matter for pointsprite
    //Out.TexCdCol = mul(TexCd, tTexCol);

	Out.Size = Size * (tP / Out.Pos.w ) * 200 /*(ViewportHeight * 0.66666666666)*/;
	//Out.Size = (Size * 1) * mul(Pos, tVP).z;
	
    return Out;
}

vs2psQuad VSQuad(
		float4 Pos : POSITION,
		float4 TexCd : TEXCOORD0
	) 
{
    //inititalize all fields of output struct with 0
    vs2psQuad Out = (vs2psQuad)0;
	
    //apply the tW (points at the mesh position)
    Pos.xyz = mul(Pos, tW);
    //then apply the tVP
    Out.Pos = mul(Pos, tVP);

	//doesn't seem to matter for pointsprite
    //Out.TexCdCol = mul(TexCd, tTexCol);
	
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PSPointSprite(vs2psPointSprite In): COLOR
{		
	return tex2D( SampCol, In.TexCdCol );
}

float4 PSQuad(vs2psQuad In): COLOR
{
	return tex2D( SampCol, In.TexCdCol );
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------


//This technique is too simple, and gives errors with textures containing an alpha channel 
technique TSimplePointSprite
{	
    pass P0
    {
		FillMode = POINT;
		PointScaleEnable = true;
		PointSpriteEnable = true;

        VertexShader = compile vs_2_0 VSPointSprite();
        PixelShader = compile ps_2_0 PSPointSprite();
    }
}

technique TSimpleQuad
{	
    pass P0
    {
        VertexShader = compile vs_2_0 VSQuad();
        PixelShader = compile ps_2_0 PSQuad();
    }
}
